package com.qmth.distributed.print.api.obe;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.reflect.TypeToken;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.core.rateLimit.annotation.RateLimit;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.service.BasicExamStudentService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.obe.been.dto.ExamParamDto;
import com.qmth.teachcloud.obe.been.dto.TCFinalScoreDto;
import com.qmth.teachcloud.obe.been.excel.PaperStructDto;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.entity.TCFinalScore;
import com.qmth.teachcloud.obe.entity.TCPaperStruct;
import com.qmth.teachcloud.obe.service.TCFinalScoreService;
import com.qmth.teachcloud.obe.service.TCPaperStructService;
import com.qmth.teachcloud.obe.service.TRBasicInfoService;
import io.swagger.annotations.*;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 期末考试成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-成绩管理-期末考试成绩Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TCFinalScoreController {
    private final static Logger log = LoggerFactory.getLogger(TCFinalScoreController.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    TCFinalScoreService tcFinalScoreService;

    @Resource
    MarkQuestionService markQuestionService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    TCPaperStructService tcPaperStructService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    BasicExamStudentService basicExamStudentService;

    @ApiOperation(value = "导入期末成绩-模板下载")
    @RequestMapping(value = "/final_score/template_download", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void finalScoreTemplateDownload(@ApiParam(value = "考试id") @RequestParam(required = false) Long examId,
                                           @ApiParam(value = "科目id", required = true) @RequestParam Long courseId,
                                           @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                           @ApiParam(value = "培养方案id", required = true) @RequestParam Long cultureProgramId) throws IOException {
        tcFinalScoreService.getLock(cultureProgramId, courseId, paperNumber, examId);
        TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(cultureProgramId, courseId, paperNumber, examId);
        List<Set<ExcelField>> excelFieldRowSet = new ArrayList<>();
        List<TCFinalScore> finalScoreList = tcFinalScoreService.queryFinalScore(cultureProgramId, courseId, paperNumber, examId);
        if (CollectionUtils.isNotEmpty(finalScoreList)) {
            finalScoreList.stream().peek(s -> {
                        Set<ExcelField> excelFieldList = new LinkedHashSet<>();
                        excelFieldList.addAll(new ArrayList<>(Arrays.asList(new ExcelField("studentCode", "学号", s.getStudentCode(), true), new ExcelField("name", "姓名", s.getName(), true), new ExcelField("score", "成绩", Objects.nonNull(s.getScore()) ? s.getScore().toString() : null, true))));
                        excelFieldRowSet.add(excelFieldList);
                    }
            ).collect(Collectors.toList());
        } else {
            Set<ExcelField> excelFieldList = new LinkedHashSet<>();
            excelFieldList.addAll(new ArrayList<>(Arrays.asList(new ExcelField("studentCode", "学号", true), new ExcelField("name", "姓名", true), new ExcelField("score", "成绩", true))));
            excelFieldRowSet.add(excelFieldList);
        }
        if (Objects.isNull(tcPaperStruct) || Objects.isNull(tcPaperStruct.getPaperStruct())) {
            List<MarkQuestion> markQuestionList = markQuestionService.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
            if (CollectionUtils.isEmpty(markQuestionList)) {
                throw ExceptionResultEnum.ERROR.exception("未找到试卷结构");
            }
            for (int i = 0; i < markQuestionList.size(); i++) {
                MarkQuestion markQuestion = markQuestionList.get(i);
                for (int y = 0; y < excelFieldRowSet.size(); y++) {
                    Set<ExcelField> excelFieldSet = excelFieldRowSet.get(y);
                    excelFieldSet.add(new ExcelField("key" + (i + 1), markQuestion.getMainNumber() + "-" + markQuestion.getSubNumber(), true));
                }
            }
        } else {
            List<PaperStructDto> paperStructDtoList = GsonUtil.fromJson(tcPaperStruct.getPaperStruct(), new TypeToken<List<PaperStructDto>>() {
            }.getType());
            for (int i = 0; i < paperStructDtoList.size(); i++) {
                PaperStructDto paperStructDto = paperStructDtoList.get(i);
                for (int y = 0; y < excelFieldRowSet.size(); y++) {
                    Set<ExcelField> excelFieldSet = excelFieldRowSet.get(y);
                    excelFieldSet.add(new ExcelField("key" + (i + 1), paperStructDto.getMainNumber() + "-" + paperStructDto.getSubNumber(), true));
                }
            }
        }
        printCommonService.scoreDownLoadExaminationTemplate("1、所有字段均为必填字段；\n" +
                "2、期末成绩按总分，和小题分录入，题号按大题号-小题号方式录入，如下，1-1表示第一大题第一小题，2-1表示第二大题第一小题；\n" +
                "3、请不要删除此行，也不要删除模板中的任何列；\n" +
                "4、使用前请先删除样例数据。", "sheet1", "期末成绩导入模版", excelFieldRowSet);
    }

    @ApiOperation(value = "导入期末成绩")
    @RequestMapping(value = "/final_score/import", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.IMPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    public Result finalScoreImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                   @ApiParam(value = "考试id") @RequestParam(required = false) Long examId,
                                   @ApiParam(value = "课程id", required = true) @RequestParam Long courseId,
                                   @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                   @ApiParam(value = "培养方案id", required = true) @RequestParam Long cultureProgramId) {
        tcFinalScoreService.getLock(cultureProgramId, courseId, paperNumber, examId);
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.IMPORT + cultureProgramId + "_" + courseId + "_" + paperNumber + "_" + examId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_IMPORT_EXAM_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在导入数据，请稍候再试！");
        }
        Map<String, String> map = null;
        try {
            map = tcFinalScoreService.finalScoreExcelImport(file, examId, courseId, paperNumber, cultureProgramId);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "同步期末成绩选择试卷")
    @RequestMapping(value = "/final_score/choose_paper", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "同步成功", response = EditResult.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.SYNC)
    public Result finalScoreSyncChoosePaper(@ApiParam(value = "考试id") @RequestParam(required = false) Long examId,
                                            @ApiParam(value = "科目编码") @RequestParam(required = false) Long courseId) {
        BasicCourse basicCourse = basicCourseService.getById(courseId);
        Objects.requireNonNull(basicCourse, "未找到课程信息");
        Set<ExamParamDto> examParamDtoList = new LinkedHashSet<>();
        List<MarkPaper> markPaperList = markPaperService.list(new QueryWrapper<MarkPaper>().lambda().eq(MarkPaper::getExamId, examId).eq(MarkPaper::getCourseId, basicCourse.getId()).eq(MarkPaper::getStatus, MarkPaperStatus.FINISH));
        if (CollectionUtils.isEmpty(markPaperList)) {
            List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().select(" distinct exam_id as examId,paper_number as paperNumber").lambda().eq(BasicExamStudent::getExamId, examId).eq(BasicExamStudent::getCourseId, basicCourse.getId()));
            if (!CollectionUtils.isEmpty(basicExamStudentList)) {
                basicExamStudentList.stream().peek(s -> examParamDtoList.add(new ExamParamDto(s.getExamId(), s.getPaperNumber()))).collect(Collectors.toList());
            }
        } else {
            markPaperList.stream().peek(s -> examParamDtoList.add(new ExamParamDto(s.getExamId(), s.getPaperNumber()))).collect(Collectors.toList());
        }
        return ResultUtil.ok(examParamDtoList);
    }

    @ApiOperation(value = "同步期末成绩")
    @RequestMapping(value = "/final_score/sync", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "同步成功", response = EditResult.class)})
    @OperationLogDetail(operationType = OperationTypeEnum.SYNC)
    @Aac(rateLimit = @RateLimit(count = 1, period = 1000L))
    public Result finalScoreSync(@ApiParam(value = "考试id") @RequestParam(required = false) Long examId,
                                 @ApiParam(value = "课程id", required = true) @RequestParam Long courseId,
                                 @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                 @ApiParam(value = "培养方案id", required = true) @RequestParam Long cultureProgramId) {
        tcFinalScoreService.getLock(cultureProgramId, courseId, paperNumber, examId);
        String lockKey = SystemConstant.REDIS_FINAL_SCORE_DATA_PREFIX + SystemConstant.SYNC + cultureProgramId + "_" + courseId + "_" + paperNumber + "_" + examId;
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_IMPORT_EXAM_DATA_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在同步数据，请稍候再试！");
        }
        Map<String, String> map = null;
        try {
            map = tcFinalScoreService.finalScoreSync(examId, courseId, paperNumber, cultureProgramId);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "期末成绩列表")
    @RequestMapping(value = "/final_score/list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = TCFinalScore.class)})
    public Result finalScoreList(@ApiParam(value = "考试id") @RequestParam(required = false) Long examId,
                                 @ApiParam(value = "试卷编号") @RequestParam(required = false) String paperNumber,
                                 @ApiParam(value = "培养方案id", required = true) @RequestParam Long cultureProgramId,
                                 @ApiParam(value = "课程id", required = true) @RequestParam Long courseId,
                                 @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                 @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) throws IOException {
        return ResultUtil.ok(tcFinalScoreService.finalScoreList(new Page<>(pageNumber, pageSize), examId, paperNumber, cultureProgramId, courseId));
    }

    @ApiOperation(value = "期末成绩编辑")
    @RequestMapping(value = "/final_score/edit", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.SAVE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = TCFinalScore.class)})
    public Result finalScoreEdit(@ApiParam(value = "主键", required = true) @RequestParam Long id) throws IOException {
        TCFinalScore tcFinalScore = tcFinalScoreService.getById(id);
        Objects.requireNonNull(tcFinalScore, "未找到期末成绩信息");
        return ResultUtil.ok(tcFinalScoreService.getById(id));
    }

    @ApiOperation(value = "期末成绩保存")
    @RequestMapping(value = "/final_score/save", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存", response = EditResult.class)})
    @Transactional
    public Result finalScoreSave(@ApiParam(value = "期末考试成绩信息", required = true) @Valid @RequestBody TCFinalScore
                                         tcFinalScore, BindingResult bindingResult) throws IOException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        TCFinalScore tcFinalScoreDb = tcFinalScoreService.getById(tcFinalScore.getId());
        Objects.requireNonNull(tcFinalScoreDb, "未找到期末成绩信息");

        tcFinalScoreService.getLock(tcFinalScoreDb.getCultureProgramId(), tcFinalScoreDb.getCourseId(), tcFinalScoreDb.getPaperNumber(), tcFinalScoreDb.getExamId());

        List<TCFinalScoreDto> tcFinalScoreDtoList = GsonUtil.fromJson(tcFinalScore.getScoreDetail(), new TypeToken<List<TCFinalScoreDto>>() {
        }.getType());
        double score = tcFinalScoreDtoList.stream().mapToDouble(e -> Double.parseDouble(e.getScore())).sum();
        if (score > 100d) {
            throw ExceptionResultEnum.ERROR.exception("分数总分不能大于试卷总分");
        }

        TCPaperStruct tcPaperStruct = tcPaperStructService.queryPaperStruct(tcFinalScoreDb.getCultureProgramId(), tcFinalScoreDb.getCourseId(), tcFinalScoreDb.getPaperNumber(), tcFinalScoreDb.getExamId());
        Objects.requireNonNull(tcPaperStruct, "未找到试卷结构");

        List<PaperStructDimensionResult> paperStructDimensionResultList = null;
        if (Objects.nonNull(tcPaperStruct.getPaperStructDimension())) {
            paperStructDimensionResultList = GsonUtil.fromJson(tcPaperStruct.getPaperStructDimension(), new TypeToken<List<PaperStructDimensionResult>>() {
            }.getType());
        } else {
            List<PaperStructDto> paperStructDtoList = GsonUtil.fromJson(tcPaperStruct.getPaperStruct(), new TypeToken<List<PaperStructDto>>() {
            }.getType());
            paperStructDimensionResultList = new ArrayList<>(paperStructDtoList.size());
            for (PaperStructDto paperStructDto : paperStructDtoList) {
                paperStructDimensionResultList.add(new PaperStructDimensionResult(paperStructDto.getMainNumber(), paperStructDto.getSubNumber(), tcPaperStruct.getExamId(), tcPaperStruct.getPaperNumber(), new BigDecimal(paperStructDto.getScore())));
            }
        }
        Map<String, PaperStructDimensionResult> paperStructResultMap = paperStructDimensionResultList.stream().collect(Collectors.toMap(k -> k.getMainNumber() + "-" + k.getSubNumber(), Function.identity(), (dto1, dto2) -> dto1));

        for (TCFinalScoreDto tcFinalScoreDto : tcFinalScoreDtoList) {
            if (paperStructResultMap.containsKey(tcFinalScoreDto.getName().trim())) {
                PaperStructDimensionResult paperStructDimensionResult = paperStructResultMap.get(tcFinalScoreDto.getName().trim());
                BigDecimal paperScore = paperStructDimensionResult.getScore();
                BigDecimal importScore = new BigDecimal(tcFinalScoreDto.getScore());
                if (importScore.compareTo(paperScore) == 1) {
                    throw ExceptionResultEnum.ERROR.exception("输入分数大于题号" + tcFinalScoreDto.getName() + "的总分");
                }
            }
        }

        TCFinalScore tcFinalScoreSource = new TCFinalScore();
        BeanUtils.copyProperties(tcFinalScoreDb, tcFinalScoreSource);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        tcFinalScoreDb.updateInfo(tcFinalScore, sysUser.getId());

        if (!tcFinalScoreSource.equals(tcFinalScoreDb)) {
            trBasicInfoService.clearReportData(tcFinalScoreDb.getCultureProgramId(), tcFinalScoreDb.getCourseId(), tcFinalScoreDb.getPaperNumber(), tcFinalScoreDb.getExamId(), false);
            tcFinalScoreService.updateById(tcFinalScoreDb);
        }
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "期末成绩启用/禁用")
    @RequestMapping(value = "/final_score/enable", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    @Transactional
    public Result finalScoreEnable(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                   @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) {
        TCFinalScore tcFinalScoreDb = tcFinalScoreService.getById(id);
        Objects.requireNonNull(tcFinalScoreDb, "未找到期末成绩信息");

        tcFinalScoreService.getLock(tcFinalScoreDb.getCultureProgramId(), tcFinalScoreDb.getCourseId(), tcFinalScoreDb.getPaperNumber(), tcFinalScoreDb.getExamId());

        TCFinalScore tcFinalScoreSource = new TCFinalScore();
        BeanUtils.copyProperties(tcFinalScoreDb, tcFinalScoreSource);
        tcFinalScoreDb.setEnable(enable);
        if (!tcFinalScoreSource.equals(tcFinalScoreDb)) {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            tcFinalScoreDb.updateInfo(sysUser.getId());
            trBasicInfoService.clearReportData(tcFinalScoreDb.getCultureProgramId(), tcFinalScoreDb.getCourseId(), tcFinalScoreDb.getPaperNumber(), tcFinalScoreDb.getExamId(), false);
            tcFinalScoreService.updateById(tcFinalScoreDb);
        }
        return ResultUtil.ok(true);
    }
}
