package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.dto.BasicCourseExportDto;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.dto.excel.BasicCourseImportDto;
import com.qmth.teachcloud.common.bean.params.BasicCourseParams;
import com.qmth.teachcloud.common.bean.result.BasicCourseResult;
import com.qmth.teachcloud.common.bean.result.SysUserResult;
import com.qmth.teachcloud.common.bean.result.TeachCourseSelectResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicTeachCourse;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Resource
    private BasicCourseMapper basicCourseMapper;

    @Resource
    private SysOrgService sysOrgService;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    private BasicTeachCourseService basicTeachCourseService;

    @Resource
    private SysUserService sysUserService;

    @Override
    public List<BasicCourse> listCoursesByUserId(Long userId) {
        return this.baseMapper.listCoursesByUserId(userId);
    }

    @Override
    public BasicCourse findByCourseCode(String courseCode, Long schoolId) {
        BasicCourse basicCourse = this.getOne(
                new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getCode, courseCode).eq(BasicCourse::getSchoolId, schoolId));
        if (Objects.isNull(basicCourse)) {
            throw ExceptionResultEnum.ERROR.exception("未找到课程编号为[" + courseCode + "]的基础课程");
        }
        return basicCourse;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createCourse(Long schoolId, String courseCode, String courseName) {
        // 向'basic_course'表新增数据 - 教研分析用
        List<BasicCourse> basicCourseList = this.list(
                new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode));
        if (basicCourseList.size() == 0) {
            BasicCourse basicCourse = new BasicCourse();
            basicCourse.setId(SystemConstant.getDbUuid());
            basicCourse.setCode(courseCode);
            basicCourse.setName(courseName);
            basicCourse.setSchoolId(schoolId);
            this.save(basicCourse);
        }
    }

    @Override
    public IPage<BasicCourseResult> basicCoursePage(Long belongOrgId, String courseCodeOrName, Long startCreateTime, Long endCreateTime, int pageNumber, int pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());

        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(),
                ServletUtil.getRequest().getServletPath());

        courseCodeOrName = SystemConstant.translateSpecificSign(courseCodeOrName);
        IPage<BasicCourseResult> iPage = basicCourseMapper.findBasicCoursePage(new Page<>(pageNumber, pageSize),
                belongOrgId, courseCodeOrName, startCreateTime, endCreateTime, schoolId, dpr);
        List<BasicCourseResult> list = iPage.getRecords();
        for (BasicCourseResult basicCourseResult : list) {
            Long courseId = basicCourseResult.getId();
            List<BasicTeachCourse> basicTeachCourseList = basicTeachCourseService.list(
                    new QueryWrapper<BasicTeachCourse>().lambda().eq(BasicTeachCourse::getCourseId, courseId));
            if (!CollectionUtils.isEmpty(basicTeachCourseList)) {
                Set<Long> teachIdSet = basicTeachCourseList.stream().map(BasicTeachCourse::getUserId).collect(Collectors.toSet());
                List<SysUserResult> sysUserResultList = sysUserService.findCourseSysUserResultByTeachIdSet(teachIdSet);
                basicCourseResult.setTeachList(sysUserResultList);
            }
        }
        return iPage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveBasicCourse(BasicCourseParams basicCourseParams) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        SysOrg sysOrg = sysOrgService.getById(basicCourseParams.getTeachingRoomId());
        if (sysOrg == null) {
            throw ExceptionResultEnum.ERROR.exception("开课学院[" + sysOrg.getName() + "]在组织架构中不存在");
        }
        BasicCourse basicCourse = this.getByTeachRoomIdAndCode(basicCourseParams.getTeachingRoomId(), basicCourseParams.getCourseCode());
        if (basicCourseParams.getId() == null) {
            // 校验课程代码和开课学院
            if (basicCourse != null) {
                throw ExceptionResultEnum.ERROR.exception("开课学院[" + sysOrg.getName() + "]下已存在课程代码[" + basicCourseParams.getCourseCode() + "]的课程");
            } else {
                basicCourse = new BasicCourse(sysUser.getSchoolId(), basicCourseParams.getTeachingRoomId(), basicCourseParams.getCourseCode(), basicCourseParams.getCourseName());
                basicCourse.insertInfo(sysUser.getId());
                this.save(basicCourse);
                return basicCourse.getId();
            }
        } else {
            if (basicCourse != null && !basicCourse.getId().equals(basicCourseParams.getId())) {
                throw ExceptionResultEnum.ERROR.exception("开课学院[" + sysOrg.getName() + "]下已存在课程代码[" + basicCourseParams.getCourseCode() + "]的课程");
            }
            BasicCourse editBasicCourse = this.getById(basicCourseParams.getId());
            editBasicCourse.setName(basicCourseParams.getCourseName());
            editBasicCourse.setTeachingRoomId(basicCourseParams.getTeachingRoomId());
            editBasicCourse.updateInfo(sysUser.getId());
            this.updateById(editBasicCourse);
            return editBasicCourse.getId();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicCourseBatch(List<Long> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            throw ExceptionResultEnum.ERROR.exception("请选择删除的课程");
        }

        for (Long courseId : idList) {
            BasicCourse basicCourse = this.getById(courseId);
            if (basicCourse == null) {
                continue;
            }
            // 考生管理
            int countBasicExamStudentByCourseId = this.baseMapper.countBasicExamStudentByCourseId(courseId);
            if (countBasicExamStudentByCourseId > 0) {
                throw ExceptionResultEnum.ERROR.exception("课程代码[" + basicCourse.getCode() + "]，课程名称[" + basicCourse.getName() + "]下已有考生数据，不能删除");
            }
            // 命题任务
            int countExamTaskByCourseId = this.baseMapper.countExamTaskByCourseId(courseId);
            if (countExamTaskByCourseId > 0) {
                throw ExceptionResultEnum.ERROR.exception("课程代码[" + basicCourse.getCode() + "]，课程名称[" + basicCourse.getName() + "]下已有命题数据，不能删除");
            }
        }
        // 删除课程管理中老师课程
        this.baseMapper.deleteTeachCourseByCourseIds(idList);
        return this.removeByIds(idList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicCourseBatchByQuery(Long belongOrgId, String courseName, Long startCreateTime, Long endCreateTime) {
        QueryWrapper<BasicCourse> basicCourseQueryWrapper = new QueryWrapper<>();
        if (SystemConstant.longNotNull(belongOrgId)) {
            basicCourseQueryWrapper.lambda().eq(BasicCourse::getTeachingRoomId, belongOrgId);
        }
        if (SystemConstant.strNotNull(courseName)) {
            basicCourseQueryWrapper.lambda().like(BasicCourse::getName, SystemConstant.translateSpecificSign(courseName));
        }
        if (SystemConstant.longNotNull(startCreateTime)) {
            basicCourseQueryWrapper.lambda().ge(BasicCourse::getCreateTime, startCreateTime);
        }
        if (SystemConstant.longNotNull(endCreateTime)) {
            basicCourseQueryWrapper.lambda().le(BasicCourse::getCreateTime, endCreateTime);
        }
        List<BasicCourse> basicCourseList = this.list(basicCourseQueryWrapper);
        List<Long> idList = basicCourseList.stream().map(BaseEntity::getId).distinct().collect(Collectors.toList());

        return idList.size() > 0 ? this.removeBasicCourseBatch(idList) : true;
    }

    @Override
    public void exportData(HttpServletResponse response, Long belongOrgId, String courseName, Long startCreateTime, Long endCreateTime) throws Exception {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());

        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(),
                ServletUtil.getRequest().getServletPath());

        courseName = SystemConstant.translateSpecificSign(courseName);
        List<BasicCourseExportDto> courseExportDtos = basicCourseMapper.listExportData(belongOrgId, courseName,
                startCreateTime, endCreateTime, schoolId, dpr);
        if (courseExportDtos.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("没有可导出数据");
        }
        ExcelUtil.excelExport("课程数据", BasicCourseExportDto.class, courseExportDtos, response);
    }

    @Override
    public List<TeachCourseSelectResult> listByOrgId(Long orgId, Long examId, String courseName, SysUser requestUser) {
        List<Long> temp = new ArrayList<>();
        temp.add(orgId);

        Set<Long> orgIdSet = sysOrgService.findDeepOrgIdListByOrgIdList(temp);
        List<TeachCourseSelectResult> result = new ArrayList<>();
        if (orgIdSet.size() > 0) {
            // 全校学院级机构map
            Map<Long, String> collegeMap = new HashMap<>();
            for (Long teachRoomId : orgIdSet) {
                if (!collegeMap.containsKey(teachRoomId)) {
                    SysOrg sysOrg = sysOrgService.findCollegeLevelOrgByOrgId(teachRoomId);
                    if (Objects.nonNull(sysOrg)) {
                        collegeMap.put(teachRoomId, sysOrg.getName());
                    }
                }
            }
            // 查询本次考试已选择的课程编号
            List<String> occupiedCourseCodeList = this.baseMapper.findOccupiedCourseCode(examId, requestUser.getId());
            LambdaQueryWrapper<BasicCourse> lambda = new QueryWrapper<BasicCourse>().lambda();
            lambda.in(BasicCourse::getTeachingRoomId, orgIdSet).orderByAsc(BasicCourse::getOrgId);
            if (StringUtils.isNotBlank(courseName)) {
                lambda.like(BasicCourse::getName, courseName);
            }
            result = this.list(lambda).stream().filter(e -> !occupiedCourseCodeList.contains(e.getCode())).flatMap(e -> {
                TeachCourseSelectResult teachCourseSelectResult = new TeachCourseSelectResult();
                teachCourseSelectResult.setBasicCourseId(e.getId());
                teachCourseSelectResult.setCourseName(e.getName());
                teachCourseSelectResult.setCourseCode(e.getCode());
                teachCourseSelectResult.setTeachCollegeName(collegeMap.get(e.getTeachingRoomId()));
                return Stream.of(teachCourseSelectResult);
            }).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BasicCourse> listBySchoolIdAndCode(Long schoolId, String courseCode) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId)
                .eq(BasicCourse::getCode, courseCode);
        return this.list(queryWrapper);
    }


    @Transactional
    @Override
    public void importBasicCourse(MultipartFile file) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        try {
            InputStream inputStream = file.getInputStream();
            String extension = FilenameUtils.getExtension(file.getOriginalFilename());
            ExcelReader excelReader;
            if (SystemConstant.XLS.equals(extension)) {
                excelReader = ExcelReader.create(ExcelType.XLS, inputStream, 1);
            } else if (SystemConstant.XLSX.equals(extension)) {
                excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 1);
            } else {
                throw ExceptionResultEnum.ERROR.exception("只能导入后缀为.xls或.xlsx文件");
            }
            List<BasicCourseImportDto> courseImportDtoList = excelReader.getObjectList(BasicCourseImportDto.class);
            if (CollectionUtils.isEmpty(courseImportDtoList)) {
                throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
            }
            List<BasicCourse> basicCourseList = new ArrayList<>();
            List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(sysUser.getSchoolId(), null);
            if (CollectionUtils.isEmpty(sysOrgList)) {
                throw ExceptionResultEnum.ERROR.exception("学校没有学院机构");
            }
            Map<String, Long> teachingRoomNameIdMap = sysOrgList.stream().collect(Collectors.toMap(SysOrg::getName, SysOrg::getId));
            for (BasicCourseImportDto basicCourseImportDto : courseImportDtoList) {
                String courseName = basicCourseImportDto.getCourseName();
                String courseCode = basicCourseImportDto.getCourseCode();
                String teachingRoomName = basicCourseImportDto.getTeachingRoomName();
                // 所有字段为空，跳过
                if (StringUtils.isAllBlank(courseCode, courseName, teachingRoomName)) {
                    continue;
                }
                if (StringUtils.isAnyBlank(courseCode, courseName, teachingRoomName)) {
                    throw ExceptionResultEnum.ERROR.exception("课程代码、课程名称、开课学院不能为空");
                }
                // 去空格
                courseName = StringUtils.deleteWhitespace(courseName);
                courseCode = StringUtils.deleteWhitespace(courseCode);
                teachingRoomName = StringUtils.deleteWhitespace(teachingRoomName);
                // 校验开课学院
                if (!teachingRoomNameIdMap.containsKey(teachingRoomName)) {
                    throw ExceptionResultEnum.ERROR.exception("开课学院[" + teachingRoomName + "]在学院层级不存在");
                }

                Long teachingRoomId = teachingRoomNameIdMap.get(teachingRoomName);

                // 校验课程代码和开课学院
                BasicCourse basicCourse = this.getByTeachRoomIdAndCode(teachingRoomId, courseCode);
                if (basicCourse != null) {
                    basicCourse.setName(courseName);
                    basicCourse.setUpdateId(sysUser.getId());
                    basicCourse.setUpdateTime(System.currentTimeMillis());
                    basicCourse.setEnable(true);
                } else {
                    basicCourse = new BasicCourse(sysUser.getSchoolId(), teachingRoomId, courseCode, courseName);
                    basicCourse.insertInfo(sysUser.getId());
                }
                basicCourseList.add(basicCourse);
            }
            this.saveOrUpdateBatch(basicCourseList);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("导入失败：" + e.getMessage());
        }
    }

    @Override
    public BasicCourse getByTeachRoomIdAndCode(Long teachRoomId, String courseCode) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getTeachingRoomId, teachRoomId)
                .eq(BasicCourse::getCode, courseCode);
        return this.getOne(queryWrapper);
    }
}
