package com.qmth.teachcloud.obe.been.result.report;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.obe.been.report.*;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.util.LinkedMultiValueMap;

import java.io.Serializable;
import java.util.Map;

/**
 * @Description: 报告result
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/2/19
 */
public class ReportResult implements Serializable {

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "主键")
    private Long id;

    @ApiModelProperty(value = "教学班")
    private String teachClassName;

    @ApiModelProperty(value = "公共信息")
    private ReportCommonDto commonInfo;

    @ApiModelProperty(value = "课程基本情况")
    private ReportCourseBasicInfoDto courseBasicInfo;

    @ApiModelProperty(value = "课程目标考核分布")
    private ReportCourseEvaluationSpreadDto courseEvaluationSpreadInfo;

    @ApiModelProperty(value = "课程考核成绩评价结果")
    private ReportCourseEvaluationResultDto courseEvaluationResultInfo;

    @ApiModelProperty(value = "课程目标达成评价明细结果")
    private ReportCourseEvaluationResultDetailDto courseEvaluationResultDetailInfo;

    @ApiModelProperty(value = "课程持续改进")
    private String courseSuggest;

    @ApiModelProperty(value = "课程目标达成度散点图")
    private LinkedMultiValueMap<Long, Map<String, String>> courseTargetScatterMap;

    public ReportResult() {

    }

    public ReportResult(ReportCommonDto commonInfo, ReportCourseBasicInfoDto courseBasicInfo,
                        ReportCourseEvaluationSpreadDto courseEvaluationSpreadInfo,
                        ReportCourseEvaluationResultDto courseEvaluationResultInfo,
                        ReportCourseEvaluationResultDetailDto courseEvaluationResultDetailInfo, TRBasicInfo trBasicInfo,
                        LinkedMultiValueMap<Long, Map<String, String>> courseTargetScatterMap) {
        this.commonInfo = commonInfo;
        this.courseBasicInfo = courseBasicInfo;
        this.courseEvaluationSpreadInfo = courseEvaluationSpreadInfo;
        this.courseEvaluationResultInfo = courseEvaluationResultInfo;
        this.courseEvaluationResultDetailInfo = courseEvaluationResultDetailInfo;
        this.courseSuggest = trBasicInfo.getCourseSuggest();
        this.courseTargetScatterMap = courseTargetScatterMap;
        this.teachClassName = trBasicInfo.getTeachClassName();
        this.id = trBasicInfo.getId();
    }

    public void updateInfo(String teachClassName, Long id) {
        this.teachClassName = teachClassName;
        this.id = id;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getTeachClassName() {
        return teachClassName;
    }

    public void setTeachClassName(String teachClassName) {
        this.teachClassName = teachClassName;
    }

    public LinkedMultiValueMap<Long, Map<String, String>> getCourseTargetScatterMap() {
        return courseTargetScatterMap;
    }

    public void setCourseTargetScatterMap(LinkedMultiValueMap<Long, Map<String, String>> courseTargetScatterMap) {
        this.courseTargetScatterMap = courseTargetScatterMap;
    }

    public String getCourseSuggest() {
        return courseSuggest;
    }

    public void setCourseSuggest(String courseSuggest) {
        this.courseSuggest = courseSuggest;
    }

    public ReportCommonDto getCommonInfo() {
        return commonInfo;
    }

    public void setCommonInfo(ReportCommonDto commonInfo) {
        this.commonInfo = commonInfo;
    }

    public ReportCourseBasicInfoDto getCourseBasicInfo() {
        return courseBasicInfo;
    }

    public void setCourseBasicInfo(ReportCourseBasicInfoDto courseBasicInfo) {
        this.courseBasicInfo = courseBasicInfo;
    }

    public ReportCourseEvaluationSpreadDto getCourseEvaluationSpreadInfo() {
        return courseEvaluationSpreadInfo;
    }

    public void setCourseEvaluationSpreadInfo(ReportCourseEvaluationSpreadDto courseEvaluationSpreadInfo) {
        this.courseEvaluationSpreadInfo = courseEvaluationSpreadInfo;
    }

    public ReportCourseEvaluationResultDto getCourseEvaluationResultInfo() {
        return courseEvaluationResultInfo;
    }

    public void setCourseEvaluationResultInfo(ReportCourseEvaluationResultDto courseEvaluationResultInfo) {
        this.courseEvaluationResultInfo = courseEvaluationResultInfo;
    }

    public ReportCourseEvaluationResultDetailDto getCourseEvaluationResultDetailInfo() {
        return courseEvaluationResultDetailInfo;
    }

    public void setCourseEvaluationResultDetailInfo(ReportCourseEvaluationResultDetailDto courseEvaluationResultDetailInfo) {
        this.courseEvaluationResultDetailInfo = courseEvaluationResultDetailInfo;
    }
}
