package com.qmth.teachcloud.obe.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.obe.been.dto.FinalScoreDto;
import com.qmth.teachcloud.obe.been.result.FinalScoreResult;
import com.qmth.teachcloud.obe.been.result.ObeScoreResult;
import com.qmth.teachcloud.obe.entity.TCFinalScore;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 期末考试成绩表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TCFinalScoreService extends IService<TCFinalScore> {

    /**
     * 导入期末成绩excel
     *
     * @param file
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     * @throws IOException
     */
    public Map<String, String> finalScoreExcelImport(MultipartFile file, Long examId, Long courseId, String paperNumber, Long cultureProgramId) throws IOException;

    /**
     * 期末成绩列表（分页）
     *
     * @param iPage
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @return
     */
    public IPage<TCFinalScore> finalScoreList(IPage<Map> iPage, Long examId, String paperNumber, Long cultureProgramId, Long courseId);

    /**
     * 查找期末成绩分数图
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param teachClassName
     * @param userId
     * @return
     */
    public FinalScoreDto finalScoreScoreOverView(Long examId, String paperNumber, Long cultureProgramId, Long courseId, String teachClassName, Long userId);

    /**
     * 查找期末成绩分数区间
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param start
     * @param end
     * @param teachClassName
     * @param userId
     * @return
     */
    public int getCountByScoreRange(Long examId, String paperNumber, Long cultureProgramId, Long courseId, Double start, Double end, String teachClassName, Long userId);

    /**
     * 查找考生成绩
     *
     * @param examId
     * @param paperNumber
     * @param cultureProgramId
     * @param courseId
     * @param teachClassName
     * @param userId
     * @return
     */
    public List<FinalScoreResult> examStudentOverview(Long examId, String paperNumber, Long cultureProgramId, Long courseId, String teachClassName, Long userId);

    /**
     * 同步期末成绩
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param cultureProgramId
     * @return
     * @throws IOException
     */
    public Map<String, String> finalScoreSync(Long examId, Long courseId, String paperNumber, Long cultureProgramId) throws IOException;

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public List<TCFinalScore> queryFinalScore(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取期末考试同步锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getFinalScoreSyncLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取期末考试导入锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getFinalScoreImportLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 成绩管理列表
     *
     * @param iPage
     * @param cultureProgramId 培养方案id
     * @param semesterId       学期id
     * @param courseId         课程id
     * @return
     */
    public IPage<ObeScoreResult> scoreList(IPage<Map> iPage, Long cultureProgramId, Long semesterId, Long courseId);

    /**
     * 获取成绩管理需要的锁
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     */
    public void getLock(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 校验方案下课程是否已导入成绩
     *
     * @param cultureProgramId 培养方案ID
     * @param courseId         课程ID
     */
    void checkUsualOrFinalScoreExist(Long cultureProgramId, Long courseId);
}
