package com.qmth.teachcloud.obe.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.deepoove.poi.data.ChartMultiSeriesRenderData;
import com.deepoove.poi.data.TableRenderData;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.obe.been.dto.CourseWeightDto;
import com.qmth.teachcloud.obe.been.obe.CourseSuggestDto;
import com.qmth.teachcloud.obe.been.report.*;
import com.qmth.teachcloud.obe.been.result.ObeCourseWeightResult;
import com.qmth.teachcloud.obe.been.result.report.PaperStructDimensionResult;
import com.qmth.teachcloud.obe.been.result.report.word.common.CourseReportBean;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuCourseReportBean;
import com.qmth.teachcloud.obe.been.result.report.word.custom.XagyuCourseSuggestDto;
import com.qmth.teachcloud.obe.entity.ObeCourseOutline;
import com.qmth.teachcloud.obe.entity.TRBasicInfo;
import org.springframework.util.LinkedMultiValueMap;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

/**
 * <p>
 * 报告基本情况表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
public interface TRBasicInfoService extends IService<TRBasicInfo> {

    /**
     * 根据考试id/课程编码/试卷编码查询数据库数据
     *
     * @param cultureProgramId cultureProgramId
     * @param courseId         courseId
     * @param paperNumber      paperNumber
     * @param examId
     * @return
     */
    public List<TRBasicInfo> queryBasicInfo(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 获取课程目标考核分布-分数图
     *
     * @param trBasicInfo
     * @param markPaper
     * @return
     */
    public ReportScoreViewDto getScoreList(TRBasicInfo trBasicInfo, MarkPaper markPaper);

    /**
     * 获取计算参数
     *
     * @param trBasicInfo
     * @param paperStructDimensionResultList
     * @param obeCourseWeightResult
     * @return
     */
    public Map<String, Object> getReportCalculateParams(TRBasicInfo trBasicInfo, List<PaperStructDimensionResult> paperStructDimensionResultList,
                                                        ObeCourseWeightResult obeCourseWeightResult);

    /**
     * 计算课程考核成绩评价结果
     *
     * @param trBasicInfo
     * @param paramsMap
     * @return
     */
    public ReportCourseEvaluationResultDto reportCalculateCourseEvaluationResult(TRBasicInfo trBasicInfo, Map<String, Object> paramsMap);

    /**
     * 获取报告
     *
     * @param markPaper
     * @param obeCourseWeightResult
     * @param cultureProgramId
     * @param courseId
     * @param examId
     * @param paperNumber
     * @return
     */
    public List<TRBasicInfo> getReportView(MarkPaper markPaper, ObeCourseWeightResult obeCourseWeightResult, Long cultureProgramId, Long courseId, Long examId, String paperNumber);

    /**
     * 构建word动态表格1-课程目标与毕业要求指标点的对应关系
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable1(List<CourseTargetWordDto> courseTargetWordDtoList);

    /**
     * 构建word动态表格2-课程目标达成考核/评价环节及目标分值
     *
     * @param courseTargetWordDtoList
     * @return
     */
    public TableRenderData buildWordTable2(List<CourseTargetWordDto> courseTargetWordDtoList);

    /**
     * 构建word动态表格5-各课程目标考核样本数据详情
     *
     * @param trBasicInfo
     * @param courseReportBean
     * @return
     */
    public TableRenderData buildWordTable5(TRBasicInfo trBasicInfo, CourseReportBean courseReportBean);

    /**
     * 构建word动态表格6
     *
     * @param map
     * @return
     */
    public TableRenderData buildWordTable6(Map<String, Object> map);

    /**
     * 构建word图形表格1
     *
     * @param map
     * @return
     */
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram1(Map<String, Object> map);

    /**
     * 构建word图形表格2
     *
     * @param map
     * @return
     */
    public ChartMultiSeriesRenderData buildCourseTargetColumnDiagram2(Map<String, Object> map);

    /**
     * 构建word图形表格3（散点图）
     *
     * @param finalScatterYMap
     * @param courseSuggestDtoList
     * @param courseTargetWebDtoList
     * @param examStudentSize
     */
    public void buildCourseTargetColumnDiagram3(LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap,
                                                List<CourseSuggestDto> courseSuggestDtoList,
                                                List<CourseTargetWebDto> courseTargetWebDtoList,
                                                int examStudentSize);

    /**
     * 根据考试id/课程编码/试卷编码删除数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param examId
     * @return
     */
    public Boolean remove(Long cultureProgramId, Long courseId, String paperNumber, Long examId);

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @param vaildWeight
     * @return
     */
    public ObeCourseWeightResult findCourseWeightResultRmi(Long obeCourseOutlineId, boolean vaildWeight);

    /**
     * 查找课程评价值
     *
     * @param obeCourseOutlineId
     * @return
     */
    public List<CourseWeightDto> findCourseWeightDtoRmi(Long obeCourseOutlineId);

    /**
     * 清除报告数据
     *
     * @param cultureProgramId
     * @param courseId
     * @param paperNumber
     * @param clearDimension
     * @param examId
     */
    public void clearReportData(Long cultureProgramId, Long courseId, String paperNumber, Long examId, boolean clearDimension);

    /**
     * 报表基础信息
     *
     * @param trBasicInfo
     * @param obeCourseOutline
     * @param markPaper
     * @param obeCourseWeightResult
     * @return
     */
    public ReportCourseBasicInfoDto setBasicInfo(TRBasicInfo trBasicInfo, ObeCourseOutline obeCourseOutline, MarkPaper markPaper, ObeCourseWeightResult obeCourseWeightResult);

    /**
     * 获取散点图值
     *
     * @param scatterYMap
     * @param studentCode
     * @param reportExamStudentTargetDto
     * @return
     */
    public Map<Long, ConcurrentMap<String, String>> getScatterValue(Map<Long, ConcurrentMap<String, String>> scatterYMap, String studentCode, ReportExamStudentTargetDto reportExamStudentTargetDto);

    /**
     * 获取最终散点图值
     *
     * @param scatterYMap
     * @return
     */
    public LinkedMultiValueMap<Long, Map<String, String>> getFinalScatterValue(Map<Long, ConcurrentMap<String, String>> scatterYMap);

    /**
     * 构建公共课程目标达成度模板
     *
     * @param trBasicInfo
     * @return
     */
    public CourseReportBean buildCommonCourseDegreeReport(TRBasicInfo trBasicInfo);

    /**
     * 构建西安工业大学课程目标达成度模板
     *
     * @param trBasicInfo
     * @return
     */
    public XagyuCourseReportBean buildXagyuCourseDegreeReport(TRBasicInfo trBasicInfo);

    /**
     * 构建西安工业大学课程目标达成度-各课程目标考核样本数据详情
     *
     * @param trBasicInfo
     * @param xagyuCourseReportBean
     * @return
     */
    public TableRenderData buildXagyuWordTable1(TRBasicInfo trBasicInfo, XagyuCourseReportBean xagyuCourseReportBean);

    /**
     * 构建西安工业大学课程目标达成度（散点图）
     *
     * @param finalScatterYMap
     * @param courseSuggestDtoList
     * @param courseTargetWebDtoList
     * @param examStudentSize
     */
    public void buildXagyuCourseTargetColumnDiagram1(LinkedMultiValueMap<Long, Map<String, String>> finalScatterYMap,
                                                     List<XagyuCourseSuggestDto> courseSuggestDtoList,
                                                     List<CourseTargetWebDto> courseTargetWebDtoList,
                                                     int examStudentSize);

    /**
     * 构建考生成绩
     *
     * @param trBasicInfo
     * @return
     */
    public Map<String, Object> buildExamStudentScoreMap(TRBasicInfo trBasicInfo);

    /**
     * 构建分数图
     *
     * @param scoreList
     * @return
     */
    public Map<String, Object> buildScoreOtherMap(List<BigDecimal> scoreList);

    /**
     * 构建西安工业大学课程目标达成度-分数图
     *
     * @param scoreListMap
     * @return
     */
    public Map<String, Object> buildXagyuScoreOtherMap(LinkedMultiValueMap<String, BigDecimal> scoreListMap);

    /**
     * 构建西安工业大学课程目标达成度-课程目标的评价标准
     *
     * @param map
     * @return
     */
    public TableRenderData buildXagyuWordTable2(Map<String, Object> map);

}
